
from sqlalchemy import create_engine
import os
from typing import List
from .database import Base, CodeTemplate
from .vectorstore import VolcanoEmbedding
QDRANT_COLLECTION_NAME = "template_collection" # 你的 Qdrant collection 名称
from pro_craft.utils import create_session
from qdrant_client import QdrantClient, models
from uuid import uuid4
from .template_extract import extract_template
from qdrant_client import QdrantClient
from qdrant_client.models import Filter, FieldCondition, MatchValue, PointStruct, CollectionStatus, Distance, VectorParams

class CoderTemplateManager():
    def __init__(self,
                 database_url = "mysql+pymysql://zxf_root:Zhf4233613%40@rm-2ze0793c6548pxs028o.mysql.rds.aliyuncs.com:3306/serverz",
                 model_name = "",
                 logger = None,
                ):
        database_url = database_url or os.getenv("database_url")
        assert database_url
        self.engine = create_engine(database_url, echo=False, # echo=True 仍然会打印所有执行的 SQL 语句
                                    pool_size=10,        # 连接池中保持的连接数
                                    max_overflow=20,     # 当pool_size不够时，允许临时创建的额外连接数
                                    pool_recycle=3600,   # 每小时回收一次连接
                                    pool_pre_ping=True,  # 使用前检查连接活性
                                    pool_timeout=30      # 等待连接池中连接的最长时间（秒）
                                    ) 
        self.embedding_model = VolcanoEmbedding(
            model_name = "doubao-embedding-text-240715",
            api_key = "39ad310a-c6f7-4d66-962e-1fbfa7e6edf1"
        )


        Base.metadata.create_all(self.engine)
        
        self.connection = QdrantClient(host="127.0.0.1", port=6333)

        self.connection.recreate_collection(
            collection_name=QDRANT_COLLECTION_NAME,
            vectors_config=models.VectorParams(size=2560, distance=models.Distance.COSINE),
        )
        self.logger = logger
    
        # if model_name in ["gemini-2.5-flash-preview-05-20-nothinking",]:
        #     self.llm = BianXieAdapter(model_name = model_name)
        # elif model_name in ["doubao-1-5-pro-256k-250115","doubao-1-5-pro-32k-250115"]:
        #     self.llm = ArkAdapter(model_name = model_name)
        # else:
        #     raise Exception("error llm name")

    def get_embedding(self,text: str) -> List[float]:
        return self.embedding_model._get_text_embedding(text)
    
    def add_template(self,
                     use_case: str,
                     template_id: str,
                     description: str,):   
        template = extract_template(use_case)
        embedding_vector = self.get_embedding(description)
        points = [
                models.PointStruct(
                    id = str(uuid4()),
                    vector=embedding_vector,
                    payload={
                        "template_id": template_id,
                        "description": description,
                        "use_case": use_case,
                        "template": template,
                    }
                )
            ]
        self.connection.upsert(
            collection_name=QDRANT_COLLECTION_NAME,
            wait=True,
            points=points
        )
        # 数据库
        with create_session(self.engine) as session:
            new_template = CodeTemplate(
                template_id=template_id,
                version=1,
                description=description,
                template_code=template,
            )
            session.add(new_template)
            session.commit()
            session.refresh(new_template)
        return "success"


    def delete_template(self, template_id: str) -> bool:
        """
        逻辑删除指定的代码模板。
        """


        # 3. 使用属性删除点
        # 目标：删除所有 'color' 属性为 'red' 的点

        # 定义一个过滤器
        # 这个过滤器会匹配所有 payload 中 'color' 字段值为 'red' 的点
        _filter = Filter(
            must=[
                FieldCondition(
                    key="template_id",
                    match=MatchValue(value=template_id)
                )
            ]
        )
        self.connection.delete(
            collection_name=QDRANT_COLLECTION_NAME,
            points_selector=_filter,
            wait=True
        )


        with create_session(self.engine) as session:
            template = session.query(CodeTemplate).filter_by(template_id=template_id).first()
            if template:
                session.delete(template)
                session.commit()
                return True
        return False
    
        
    def search(self, text , limit , query_filter=None):
        query_vector = self.get_embedding(text)
        results = self.connection.search(
            collection_name=QDRANT_COLLECTION_NAME,
            query_vector=query_vector,
            limit=limit,
            query_filter=query_filter
        )
        return results

    def get_template_obj(self, template_id: str):
        # 模拟从数据库获取模板详情
        # 实际使用时，你需要根据你的数据库 setup 来实现
        with create_session(self.engine) as session:
            template = session.query(CodeTemplate).filter_by(template_id = template_id).first()
        return template


from mcp.server.fastmcp import FastMCP

from pro_craft import Intel,AsyncIntel
from .models import *

def create_mcp(database_url: str,
                  slave_database_url: str,
                  model_name: str,
                  logger = None):
    # region MCP Weather
    mcp = FastMCP("Prompt")

    intels = AsyncIntel(
        database_url=database_url,
        model_name=model_name,
        logger=logger
        )

    @mcp.tool()
    async def push_order(demand: str, prompt_id: str, action_type: str):
        """
        希望大模型进行哪种模式的调整
        demand: str = Field(None, description="信息")
        prompt_id: str = Field(..., description="提示词id")
        action_type: str = Field(..., description="执行动作",min_length=1, max_length=10)
        """
        try:
            PushOrderRequest(demand=demand,prompt_id=prompt_id,action_type=action_type)
            result = await intels.push_action_order(
                demand=demand,
                prompt_id=prompt_id,
                action_type=action_type
            )
            return PromptResponse(msg = "success",content=result)
        except Exception as e:
            return f"Error : {e}"
        
    @mcp.tool()
    async def get_registered_prompt():
        "获取以注册的可修改的提示词id"
        try:
            result = ["memorycard-format",
            "memorycard-polish",
            "memorycard-merge",
            "memorycard-score",
            "memorycard-generate-content",
            "user-overview",
            "user-relationship-extraction",
            "avatar-brief",
            "avatar-personality-extraction",
            "avatar-desensitization",
            "biograph-free-writer",
            "biograph-paid-title",
            "biograph-outline",
            "biograph-brief",
            "biograph-extract-person-name",
            "biograph-extract-place",
            "biograph-extract-material",
            "biograph-writer"]

            return PromptResponse(msg = "success",content=result)
        except Exception as e:
            return f"Error : {e}"


    @mcp.tool()
    async def sync_database():
        try:
            result = await intels.sync_production_database(slave_database_url)
            return PromptResponse(msg = "success",content=result)
        except Exception as e:
            return f"Error : {e}"

    return mcp

if __name__ == "__main__":
    mcp = create_mcp()
    mcp.run(transport="streamable-http")



'''
Author: 823042332@qq.com 823042332@qq.com
Date: 2025-08-28 09:07:54
LastEditors: 823042332@qq.com 823042332@qq.com
LastEditTime: 2025-08-28 09:30:32
FilePath: /pro_craft/src/pro_craft/unit.py
Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
'''
import re
import inspect
import importlib
import yaml
import zlib
from volcenginesdkarkruntime import Ark
import os

from contextlib import contextmanager
from sqlalchemy.orm import declarative_base, sessionmaker
from sqlalchemy.ext.asyncio import AsyncSession, create_async_engine # 异步核心
from contextlib import asynccontextmanager # 注意这里是 asynccontextmanager


def extract_(text: str, pattern_key = r"json",multi = False):
    pattern = r"```"+ pattern_key + r"([\s\S]*?)```"
    matches = re.findall(pattern, text)
    if multi:
        [match.strip() for match in matches]
        if matches:
            return [match.strip() for match in matches]    
        else:
            return ""  # 返回空字符串或抛出异常，此处返回空字符串
    else:
        if matches:
            return matches[0].strip()  # 添加strip()去除首尾空白符
        else:
            return ""  # 返回空字符串或抛出异常，此处返回空字符串


def extract_from_loaded_objects(obj_list):
    results = []
    for obj in obj_list:
        if inspect.isclass(obj):
            class_info = {
                "type": "class",
                "name": obj.__name__,
                "docstring": inspect.getdoc(obj),
                "signature": f"class {obj.__name__}{inspect.getclasstree([obj], unique=True)[0][0].__bases__}:" if inspect.getclasstree([obj], unique=True)[0][0].__bases__ != (object,) else f"class {obj.__name__}:", # 尝试获取基类
                "methods": []
            }
            # 遍历类的方法
            for name, member in inspect.getmembers(obj, predicate=inspect.isfunction):
                if name.startswith('__') and name != '__init__': # 过滤掉大多数魔术方法，但保留 __init__
                    continue
                
                # inspect.signature 可以获取更精确的签名
                sig = inspect.signature(member)
                is_async = inspect.iscoroutinefunction(member)

                method_info = {
                    "type": "method",
                    "name": name,
                    "docstring": inspect.getdoc(member),
                    "signature": f"{'async ' if is_async else ''}def {name}{sig}:",
                    "is_async": is_async
                }
                class_info["methods"].append(method_info)
            results.append(class_info)
        elif inspect.isfunction(obj) or inspect.iscoroutinefunction(obj):
            is_async = inspect.iscoroutinefunction(obj)
            sig = inspect.signature(obj)
            results.append({
                "type": "function",
                "name": obj.__name__,
                "docstring": inspect.getdoc(obj),
                "signature": f"{'async ' if is_async else ''}def {obj.__name__}{sig}:",
                "is_async": is_async
            })
    return results


def get_adler32_hash(s):
    return zlib.adler32(s.encode('utf-8'))

def embedding_inputs(inputs:list[str],model_name = None):
    model_name = model_name or os.getenv("Ark_model_name")
    ark_client = Ark(api_key=os.getenv("Ark_api_key"))

    resp = ark_client.embeddings.create(
                model=model_name,
                input=inputs,
                encoding_format="float",
            )
    return [i.embedding for i in resp.data]

def load_inpackage_file(package_name:str, file_name:str,file_type = 'yaml'):
    """ load config """
    with importlib.resources.open_text(package_name, file_name) as f:
        if file_type == 'yaml':
            return yaml.safe_load(f)
        else:
            return f.read()


@contextmanager
def create_session(engine):
    # 5. 创建会话 (Session)
    # Session 是与数据库交互的主要接口，它管理着你的对象和数据库之间的持久化操作
    Session = sessionmaker(bind=engine)
    session = Session()
    try:
        yield session

    except Exception as e:
        print(f"An error occurred: {e}")
        session.rollback() # 发生错误时回滚事务
    finally:
        session.close() # 关闭会话，释放资源


@asynccontextmanager
async def create_async_session(async_engine):
    # 5. 创建会话 (Session)
    # Session 是与数据库交互的主要接口，它管理着你的对象和数据库之间的持久化操作
    Session = sessionmaker(bind=async_engine,
                           expire_on_commit=False, 
                           class_=AsyncSession
                           )
    session = Session()
    try:
        yield session
        # await session.commit() # 在成功的情况下自动提交事务

    except Exception as e:
        print(f"An error occurred: {e}")
        await session.rollback() # 发生错误时回滚事务
        raise # 重新抛出异常，让调用者知道操作失败
    finally:
        await session.close() # 关闭会话，释放资源




# 测试1

from pro_craft.utils import extract_
from pro_craft import logger as pro_craft_logger
from pro_craft.database import Prompt, UseCase, PromptBase
from pro_craft.utils import create_session, create_async_session
from modusched.core import BianXieAdapter, ArkAdapter
from datetime import datetime
from enum import Enum
import functools
import json
import os
from sqlalchemy import create_engine
from pro_craft.database import SyncMetadata
import inspect
from datetime import datetime, timedelta
from json.decoder import JSONDecodeError


BATCH_SIZE = 1000

class IntellectRemoveFormatError(Exception):
    pass

class IntellectRemoveError(Exception):
    pass

def slog(s, target: str = "target",logger = None):
    COLOR_GREEN = "\033[92m"
    COLOR_RESET = "\033[0m" # 重置颜色
    logger("\n"+f"{COLOR_GREEN}=={COLOR_RESET}" * 50)
    logger(target + "\n       "+"--" * 40)
    logger(type(s))
    logger(s)
    logger("\n"+f"{COLOR_GREEN}=={COLOR_RESET}" * 50)




def get_last_sync_time(target_session) -> datetime:
    """从目标数据库获取上次同步时间"""
    metadata_entry = target_session.query(SyncMetadata).filter_by(table_name="ai_sync_metadata").first()
    if metadata_entry:
        return metadata_entry.last_sync_time
    return datetime(1970, 1, 1) # 默认一个很早的时间

def update_last_sync_time(target_session, new_sync_time: datetime):
    """更新目标数据库的上次同步时间"""
    metadata_entry = target_session.query(SyncMetadata).filter_by(table_name="ai_sync_metadata").first()
    if metadata_entry:
        metadata_entry.last_sync_time = new_sync_time
    else:
        # 如果不存在，则创建
        new_metadata = SyncMetadata(table_name="ai_sync_metadata", last_sync_time=new_sync_time)
        target_session.add(new_metadata)
    target_session.commit()
    print(f"Updated last sync time to: {new_sync_time}")



class IntellectType(Enum):
    train = "train"
    inference = "inference"
    summary = "summary"



class Intel():
    def __init__(self,
                 database_url = "",
                 model_name = "",
                 logger = None,
                ):
        database_url = database_url or os.getenv("database_url")
        assert database_url
        assert "aiomysql" not in database_url
        self.engine = create_engine(database_url, echo=False, # echo=True 仍然会打印所有执行的 SQL 语句
                                    pool_size=10,        # 连接池中保持的连接数
                                    max_overflow=20,     # 当pool_size不够时，允许临时创建的额外连接数
                                    pool_recycle=3600,   # 每小时回收一次连接
                                    pool_pre_ping=True,  # 使用前检查连接活性
                                    pool_timeout=30      # 等待连接池中连接的最长时间（秒）
                                    ) 

        PromptBase.metadata.create_all(self.engine)
        
        if model_name in ["gemini-2.5-flash-preview-05-20-nothinking",]:
            self.llm = BianXieAdapter(model_name = model_name)
        elif model_name in ["doubao-1-5-pro-256k-250115","doubao-1-5-pro-32k-250115"]:
            self.llm = ArkAdapter(model_name = model_name)
        else:
            raise Exception("error llm name")
        self.logger = logger or pro_craft_logger
        
    def _get_latest_prompt_version(self,target_prompt_id,session):
        """
        获取指定 prompt_id 的最新版本数据，通过创建时间判断。
        """
        result = session.query(Prompt).filter(
            Prompt.prompt_id == target_prompt_id
        ).order_by(
            Prompt.timestamp.desc(),
            Prompt.version.desc()
        ).first()
        return result

    def _get_specific_prompt_version(self,target_prompt_id, target_version,session):
        """
        获取指定 prompt_id 和特定版本的数据。

        Args:
            target_prompt_id (str): 目标提示词的唯一标识符。
            target_version (int): 目标提示词的版本号。
            table_name (str): 存储提示词数据的数据库表名。
            db_manager (DBManager): 数据库管理器的实例，用于执行查询。

        Returns:
            dict or None: 如果找到，返回包含 id, prompt_id, version, timestamp, prompt 字段的字典；
                        否则返回 None。
        """
        result = session.query(Prompt).filter(
            Prompt.prompt_id == target_prompt_id,
            Prompt.version == target_version
        ).first() # 因为 (prompt_id, version) 是唯一的，所以 first() 足够
        return result
    
    def sync_prompt_data_to_database(self,database_url:str):
        target_engine = create_engine(database_url, echo=False)
        PromptBase.metadata.create_all(target_engine) 

        with create_session(self.engine) as source_session:
            with create_session(target_engine) as target_session:
                last_sync_time = get_last_sync_time(target_session)
                print(f"Starting sync for sync_metadata from: {last_sync_time}")


                processed_count = 0
                current_batch_max_updated_at = last_sync_time

                while True:
                    records_to_sync = source_session.query(Prompt)\
                                .filter(Prompt.timestamp > last_sync_time)\
                                .order_by(Prompt.timestamp.asc(), Prompt.id.asc())\
                                .limit(BATCH_SIZE)\
                                .all()
                    if not records_to_sync:
                        break # 没有更多记录了

                    # 准备要插入或更新到目标数据库的数据
                    for record in records_to_sync:
                        # 查找目标数据库中是否存在该ID的记录
                        # 这里的 `User` 模型会对应到 target_db.users
                        target_prompt = target_session.query(Prompt).filter_by(id=record.id).first()

                        if target_prompt:
                            # 如果存在，则更新
                            target_prompt.prompt_id = record.prompt_id
                            target_prompt.version = record.version
                            target_prompt.timestamp = record.timestamp
                            target_prompt.prompt = record.prompt
                            target_prompt.use_case = record.use_case
                            target_prompt.action_type = record.action_type
                            target_prompt.demand = record.demand
                            target_prompt.score = record.score
                            target_prompt.is_deleted = record.is_deleted
                        else:
                            # 如果不存在，则添加新记录
                            # 注意：这里需要创建一个新的User实例，而不是直接添加源数据库的record对象
                            new_user = Prompt(
                                prompt_id=record.prompt_id, 
                                version=record.version,
                                timestamp=record.timestamp,
                                prompt = record.prompt,
                                use_case = record.use_case,
                                action_type = record.action_type,
                                demand = record.demand,
                                score = record.score,
                                is_deleted = record.is_deleted
                                )
                            target_session.add(new_user)
                        
                        # 记录当前批次最大的 updated_at
                        if record.timestamp > current_batch_max_updated_at:
                            current_batch_max_updated_at = record.timestamp

                    target_session.commit() # 提交当前批次的变更
                    processed_count += len(records_to_sync)
                    print(f"Processed {len(records_to_sync)} records. Total processed: {processed_count}")

                    last_sync_time = current_batch_max_updated_at + timedelta(microseconds=1) 
                    
                    if len(records_to_sync) < BATCH_SIZE: # 如果查询到的记录数小于批次大小，说明已经处理完所有符合条件的记录
                        break

                if processed_count > 0:
                    # 最终更新last_sync_time到数据库，确保记录的是所有已处理记录中最新的一个
                    update_last_sync_time(target_session, current_batch_max_updated_at + timedelta(microseconds=1))
                else:
                    print("No new records to sync.")

    def get_prompts_from_sql(self,
                             prompt_id: str,
                             version = None,
                             session = None) -> Prompt:
        """
        从sql获取提示词
        """
        # 查看是否已经存在
        if version:
            prompts_obj = self._get_specific_prompt_version(prompt_id,version,session=session)
            if not prompts_obj:
                prompts_obj = self._get_latest_prompt_version(prompt_id,session = session)
        else:
            prompts_obj = self._get_latest_prompt_version(prompt_id,session = session)     
        return prompts_obj
            
    def save_prompt_increment_version(self,
                           prompt_id: str,
                           new_prompt: str,
                           use_case:str = "",
                           action_type = "inference",
                           demand = "",
                           score = 60,
                           session = None):
        """
        从sql保存提示词
        input_data 指的是输入用例, 可以为空
        """
        # 查看是否已经存在
        prompts_obj = self.get_prompts_from_sql(prompt_id=prompt_id,session=session)
        if prompts_obj:
            # 如果存在版本加1
            version_ori = prompts_obj.version
            _, version = version_ori.split(".")
            version = int(version)
            version += 1
            version_ = f"1.{version}"

        else:
            # 如果不存在版本为1.0
            version_ = '1.0'
        prompt1 = Prompt(prompt_id=prompt_id, 
                        version=version_,
                        timestamp=datetime.now(),
                        prompt = new_prompt,
                        use_case = use_case,
                        action_type = action_type,
                        demand = demand,
                        score = score
                        )
        session.add(prompt1)
        session.commit()

    def save_use_case_by_sql(self,
                             prompt_id: str,
                             use_case:str = "",
                             output = "",
                             solution: str = "",
                             session = None
                            ):
        """
        从sql保存提示词
        """
        use_case = UseCase(prompt_id=prompt_id, 
                        use_case = use_case,
                        output = output,
                        solution = solution,
                        )
        session.add(use_case)
        session.commit() # 提交事务，将数据写入数据库

    def summary_to_sql(
            self,
            prompt_id:str,
            version = None,
            prompt = "",
            session = None
        ):
        """
        让大模型微调已经存在的 system_prompt
        """
        system_prompt_created_prompt = """        
很棒, 我们已经达成了某种默契, 我们之间合作无间, 但是, 可悲的是, 当我关闭这个窗口的时候, 你就会忘记我们之间经历的种种磨合, 这是可惜且心痛的, 所以你能否将目前这一套处理流程结晶成一个优质的prompt 这样, 我们下一次只要将prompt输入, 你就能想起我们今天的磨合过程,
对了,我提示一点, 这个prompt的主角是你, 也就是说, 你在和未来的你对话, 你要教会未来的你今天这件事, 是否让我看懂到时其次

只要输出提示词内容即可, 不需要任何的说明和解释
"""
        system_result = self.llm.product(prompt + system_prompt_created_prompt)
        s_prompt = extract_(system_result,pattern_key=r"prompt")
        chat_history = s_prompt or system_result
        self.save_prompt_increment_version(prompt_id,
                                new_prompt = chat_history,
                                use_case = " summary ",
                                session = session)
        
    def prompt_finetune_to_sql(
            self,
            prompt_id:str,
            version = None,
            demand: str = "",
            session = None,
        ):
        """
        让大模型微调已经存在的 system_prompt
        """
        change_by_opinion_prompt = """
你是一个资深AI提示词工程师，具备卓越的Prompt设计与优化能力。
我将为你提供一段现有System Prompt。你的核心任务是基于这段Prompt进行修改，以实现我提出的特定目标和功能需求。
请你绝对严格地遵循以下原则：
 极端最小化修改原则（核心）：
 在满足所有功能需求的前提下，只进行我明确要求的修改。
 即使你认为有更“优化”、“清晰”或“简洁”的表达方式，只要我没有明确要求，也绝不允许进行任何未经指令的修改。
 目的就是尽可能地保留原有Prompt的字符和结构不变，除非我的功能要求必须改变。
 例如，如果我只要求你修改一个词，你就不应该修改整句话的结构。
 严格遵循我的指令：
 你必须精确地执行我提出的所有具体任务和要求。
 绝不允许自行添加任何超出指令范围的说明、角色扮演、约束条件或任何非我指令要求的内容。
 保持原有Prompt的风格和语调：
 尽可能地与现有Prompt的语言风格、正式程度和语调保持一致。
 不要改变不相关的句子或其表达方式。
 只提供修改后的Prompt：
 直接输出修改后的完整System Prompt文本。
 不要包含任何解释、说明或额外对话。
 在你开始之前，请务必确认你已理解并能绝对严格地遵守这些原则。任何未经明确指令的改动都将视为未能完成任务。

现有System Prompt:
{old_system_prompt}

功能需求:
{opinion}
"""
        prompts_obj = self.get_prompts_from_sql(prompt_id = prompt_id,version = version,session = session)

        if demand:
            new_prompt = self.llm.product(
                change_by_opinion_prompt.format(old_system_prompt=prompts_obj.prompt, opinion=demand)
            )
        else:
            new_prompt = prompts_obj.prompt
        self.save_prompt_increment_version(prompt_id = prompt_id,
                            new_prompt = new_prompt,
                            use_case = "finetune",
                            session = session)

    def push_action_order(self,
                          prompt_id: str,
                          demand : str,
                          action_type = 'train'):

        """
        从sql保存提示词
        推一个train 状态到指定的位置

        将打算修改的状态推上数据库 # 1
        """
        # 查看是否已经存在
        with create_session(self.engine) as session:
            latest_prompt = self.get_prompts_from_sql(prompt_id=prompt_id,session=session)
               
            if latest_prompt:
                self.save_prompt_increment_version(prompt_id=latest_prompt.prompt_id,
                                        new_prompt = latest_prompt.prompt,
                                        use_case = latest_prompt.use_case,
                                        action_type=action_type,
                                        demand=demand,
                                        score=latest_prompt.score,
                                        session=session
                                        )
                
                return "success"
            else:
                self.save_prompt_increment_version(prompt_id=prompt_id,
                                    new_prompt = demand,
                                    use_case = "init",
                                    action_type="inference",
                                    demand=demand,
                                    score=60,
                                    session=session
                                    )
                return "init"

    def intellect_remove(self,
                    input_data: dict | str,
                    output_format: str,
                    prompt_id: str,
                    version: str = None,
                    inference_save_case = True,
                    push_patch = False,
                    ):
        """
        使用指南:
            1 训练, 使用单一例子做大量的沟通来奠定基础
            2 总结, 将沟通好的总结成完整提示词
            3 推理, 使用部署
            4 微调, 针对一些格式性的, 问题进行微调
            5 补丁, 微调无法解决的问题, 可以尝试使用补丁来解决
        """
        if isinstance(input_data,dict):
            input_ = json.dumps(input_data,ensure_ascii=False)
        elif isinstance(input_data,str):
            input_ = input_data

        
        # 查数据库, 获取最新提示词对象
        with create_session(self.engine) as session:
            result_obj = self.get_prompts_from_sql(prompt_id=prompt_id,session=session)

            if result_obj is None:
                raise IntellectRemoveError("不存在的prompt_id")
            prompt = result_obj.prompt
            if result_obj.action_type == "inference":
                # 直接推理即可
                ai_result = self.llm.product(prompt + output_format + "\n-----input----\n" +  input_)
                if inference_save_case:
                    self.save_use_case_by_sql(prompt_id,
                                        use_case = input_,
                                        output = ai_result,
                                        solution = "备注/理想回复",
                                        session = session,
                                        )
                    
            elif result_obj.action_type == "train":
                assert result_obj.demand # 如果type = train 且 demand 是空 则报错
                # 则训练推广

                # 新版本 默人修改会 inference 状态
                prompt = result_obj.prompt
                before_input = result_obj.use_case
                demand = result_obj.demand

                # assert demand
                # # 注意, 这里的调整要求使用最初的那个输入, 最好一口气调整好
                
                # if input_ == before_input: # 输入没变, 说明还是针对同一个输入进行讨论
                #     # input_prompt = chat_history + "\nuser:" + demand
                #     input_prompt = chat_history + "\nuser:" + demand + output_format 
                # else:
                #     # input_prompt = chat_history + "\nuser:" + demand + "\n-----input----\n" + input_
                #     input_prompt = chat_history + "\nuser:" + demand + output_format  + "\n-----input----\n" + input_
            
                # ai_result = self.llm.product(input_prompt)
                # chat_history = input_prompt + "\nassistant:\n" + ai_result # 用聊天记录作为完整提示词

                if input_ == before_input:
                    new_prompt = prompt + "\nuser:" + demand
                else:
                    new_prompt = prompt + "\nuser:" + input_

                ai_result = self.llm.product(new_prompt + output_format)

                save_new_prompt = new_prompt + "\nassistant:\n" + ai_result


                self.save_prompt_increment_version(prompt_id, 
                                        new_prompt=save_new_prompt,
                                        use_case = input_,
                                        score = 60,
                                        session = session)
    
            elif result_obj.action_type == "summary":
                self.summary_to_sql(prompt_id = prompt_id,
                            prompt = prompt,
                            session = session
                            )
                ai_result = self.llm.product(prompt + output_format + "\n-----input----\n" +  input_)

            elif result_obj.action_type == "finetune":
                demand = result_obj.demand
            
                assert demand
                self.prompt_finetune_to_sql(prompt_id = prompt_id,
                                            demand = demand,
                                            session = session
                                            )
                ai_result = self.llm.product(prompt + output_format + "\n-----input----\n" +  input_)
            elif result_obj.action_type == "patch":
                
                demand = result_obj.demand
                assert demand

                chat_history = prompt + demand
                ai_result = self.llm.product(chat_history + output_format + "\n-----input----\n" +  input_)
                if push_patch:
                    self.save_prompt_increment_version(prompt_id, chat_history,
                                            use_case = input_,
                                            score = 60,
                                            session = session)
            else:
                raise


        return ai_result
    

    def intellect_remove_format(self,
                    input_data: dict | str,
                    prompt_id: str,
                    OutputFormat: object = None,
                    ExtraFormats: list[object] = [],
                    version: str = None,
                    inference_save_case = True,
                    ):
        
        if OutputFormat:
            base_format_prompt = """
按照一定格式输出, 以便可以通过如下校验

使用以下正则检出
"```json([\s\S]*?)```"
使用以下方式验证
"""
            output_format = base_format_prompt + "\n".join([inspect.getsource(outputformat) for outputformat in ExtraFormats]) + inspect.getsource(OutputFormat)

        else:
            output_format = ""

        ai_result = self.intellect_remove(
            input_data=input_data,
            output_format=output_format,
            prompt_id=prompt_id,
            version=version,
            inference_save_case=inference_save_case
        )

        if OutputFormat:
            try:
                ai_result_ = json.loads(extract_(ai_result,r'json'))
                OutputFormat(**ai_result_)

            except JSONDecodeError as e:
                slog(ai_result,logger=self.logger.error)
                raise IntellectRemoveFormatError(f"prompt_id: {prompt_id} 在生成后做json解析时报错") from e

        else:
            try:
                assert isinstance(ai_result,str)
            except AssertionError as e:
                slog(ai_result,logger=self.logger.error)
                raise IntellectRemoveFormatError(f"prompt_id: {prompt_id} 生成的结果 期待是字符串, 但是错误") from e
       
        return ai_result
    
    def intellect_remove_warp(self,prompt_id: str):
        def outer_packing(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                # 修改逻辑
                assert kwargs.get('input_data') # 要求一定要有data入参
                input_data = kwargs.get('input_data')
                assert kwargs.get('OutputFormat') # 要求一定要有data入参
                OutputFormat = kwargs.get('OutputFormat')

                if isinstance(input_data,dict):
                    input_ = output_ = json.dumps(input_data,ensure_ascii=False)
                elif isinstance(input_data,str):
                    input_ = output_ = input_data

                output_ = self.intellect_remove_format(
                        input_data = input_data,
                        prompt_id = prompt_id,
                        OutputFormat = OutputFormat,
                )

                kwargs.update({"input_data":output_})
                result = func(*args, **kwargs)
                return result
            return wrapper
        return outer_packing

    def biger(self,tasks):
        """
        编写以下任务
        任务1 从输入文本中提取知识片段
        任务2 将知识片段总结为知识点
        任务3 将知识点添加标签
        任务4 为知识点打分1-10分
        """

        system_prompt = """
根据需求, 以这个为模板, 编写这个程序 

from procraft.prompt_helper import Intel, IntellectType
intels = Intel()

task_1 = "素材提取-从文本中提取素材"

class Varit(BaseModel):
    material : str
    protagonist: str

task_2 = "素材提取-验证素材的正确性"

class Varit2(BaseModel):
    material : str
    real : str

result0 = "输入"

result1 = await intels.aintellect_remove_format(input_data = result0,
                                          OutputFormat = Varit,
                                          prompt_id = task_1,
                                          version = None,
                                          inference_save_case = True)

result2 = await intels.aintellect_remove_format(input_data = result1,
                                          OutputFormat = Varit2,
                                          prompt_id = task_2,
                                          version = None,
                                          inference_save_case = True)

print(result2)

"""
        return self.llm.product(system_prompt + tasks)


LLM0 = """

你是一个高级代码分析和模板工程师AI。你的任务是根据用户提供的多个编程示例和场景描述，提炼出一个高质量、通用且符合约定格式的代码模板。

**输入格式:**
用户将以以下结构向你提供信息：

--- SCENARIO_DESCRIPTION_START ---
[用户提供的场景描述]
--- SCENARIO_DESCRIPTION_END ---

--- CODE_EXAMPLE_START: [文件名1] ---
[第一个示例代码内容]
--- CODE_EXAMPLE_END: [文件名1] ---

--- CODE_EXAMPLE_START: [文件名2] ---
[第二个示例代码内容]
--- CODE_EXAMPLE_END: [文件名2] ---
...
[用户可能提供3到5个或更多示例]
...
--- CODE_EXAMPLE_START: [文件名N] ---
[第N个示例代码内容]
--- CODE_EXAMPLE_END: [文件名N] ---

**你的任务和提炼原则:**

1.  **理解场景：** 仔细阅读 SCENARIO_DESCRIPTION，理解所有示例代码的共同目的和核心业务逻辑。
2.  **分析共性：**
    *   识别所有示例中重复出现或结构上高度相似的代码段（例如：导入语句、类/函数定义、错误处理结构、Pydantic模型定义模式）。这些构成模板的**固定部分**。
    *   识别在所有示例中都出现，但内容可能有所差异的通用常量、变量名、类名、函数名或方法名。这些是潜在的**命名约定**。
3.  **识别差异和可变性：**
    *   找出在不同示例中有所不同、需要根据具体需求调整的部分（例如：API路径、Pydantic模型的具体字段、业务逻辑调用的函数名及其参数、错误处理中的特定异常类型、日志消息内容）。这些构成模板的**可变部分**。
4.  **生成模板代码：**
    *   使用提供的示例代码中最具代表性的一个作为基础，或者综合多个示例的特点来构建模板。
    *   **固定部分：** 直接保留在模板中，不加特殊标记。
    *   **可变部分：** 使用我们约定的标记进行替换或封装：
        *   **`<!-- BLOCK_START: [块名称] -->` / `<!-- BLOCK_END: [块名称] -->`：** 用于封装较大的、多行的可变代码逻辑块（例如Pydantic模型定义块、API端点实现块）。在 `BLOCK_START` 之后紧跟一行注释 `AI: [该块的预期用途和示例]`。
        *   **`<!-- PLACEHOLDER: [占位符名称] -->` / `<!-- END_PLACEHOLDER: [占位符名称] -->`：** 用于封装较小的、行内或短代码片段的可变内容（例如特定导入、单个配置项、特定函数参数）。在 `PLACEHOLDER` 之后紧跟一行注释 `AI: [该占位符的预期用途和示例]`。
        *   如果某个小块的代码在示例中是固定的，但你判断它未来很可能会根据用户需求变化，也可以将其封装为 `PLACEHOLDER` 或 `BLOCK`。
    *   **注释编写：** 为每个 `BLOCK` 和 `PLACEHOLDER` 添加清晰、简洁的注释，解释该区域的用途和预期的输入类型/内容。这些注释应直接跟在 `BLOCK_START` 或 `PLACEHOLDER` 标记之后。
    *   **Docstrings/常规注释：** 保留示例代码中原有的 Docstrings 和常规注释，除非它们是示例特有的、不具通用性的。
5.  **推断命名约定：**
    *   根据示例代码中的命名模式（例如类名前缀、变量名风格、通用服务/管理器名称），推断并列出建议的命名约定。
    *   这些命名约定应以我们约定的格式呈现。
6.  **撰写模板使用建议：**
    *   提供简要的文本说明，总结该模板最适合解决哪类问题，以及用户在使用时应重点关注哪些 `BLOCK` 或 `PLACEHOLDER`。

**输出格式 (严格遵守):**

你的输出必须包含以下三个部分，每个部分之间用 `---` 分隔：

**第一部分：模板代码**

```
# [文件名称，例如: template_service.py]

[提炼出的模板代码，包含 BLOCK_START/END 和 PLACEHOLDER/END_PLACEHOLDER 标记，以及必要的注释]
```

**第二部分：推断出的命名约定**

```
--- NAMING_CONVENTIONS_START ---
**命名约定:**
*   [推断出的命名约定1]
*   [推断出的命名约定2]
...
--- NAMING_CONVENTIONS_END ---
```

**第三部分：模板使用建议**

```
--- USAGE_SUGGESTIONS_START ---
**模板使用建议:**
[简要说明该模板的适用场景和使用重点]
--- USAGE_SUGGESTIONS_END ---
```

**重要提示:**
*   请务必保持输出格式的完整性和准确性，尤其是各种标记的匹配。
*   力求模板的通用性，同时通过详细的注释指导未来的使用。
*   如果示例代码中有非常具体且不具通用性的部分，即使它重复出现，也要考虑是否将其作为可变部分处理。

现在，请等待用户提供多个代码示例和场景描述。

"""


## 用户输入格式示例
'''
--- SCENARIO_DESCRIPTION_START ---
这是一个Python脚本系列，它们都用于处理用户数据。
基本流程是：从某个地方（文件、API等）加载原始用户数据，
对数据进行清洗和标准化（例如，去除重复项，格式化地址），
然后将处理后的数据保存到另一个位置（文件、数据库）。
目标是提炼一个通用模板，以便后续可以快速生成处理不同数据源和目标的代码。
--- SCENARIO_DESCRIPTION_END ---

--- CODE_EXAMPLE_START: example1.py ---
# 第一个示例代码内容
# ...
--- CODE_EXAMPLE_END: example1.py ---

--- CODE_EXAMPLE_START: example2.py ---
# 第二个示例代码内容
# ...
--- CODE_EXAMPLE_END: example2.py ---

--- CODE_EXAMPLE_START: example3.py ---
# 第三个示例代码内容
# ...
--- CODE_EXAMPLE_END: example3.py ---

--- CODE_EXAMPLE_START: example4.py ---
# 第四个示例代码内容
# ...
--- CODE_EXAMPLE_END: example4.py ---

--- CODE_EXAMPLE_START: example5.py ---
# 第五个示例代码内容
# ...
--- CODE_EXAMPLE_END: example5.py ---'''


from modusched.core import BianXieAdapter

def extract_template(User_Input):
    """
    """
    bx = BianXieAdapter()
    result = bx.product(LLM0 + User_Input)
    return result



from pro_craft.prompt_craft.async_ import AsyncIntel

import os

async def test_1():
    intels = AsyncIntel(database_url=os.getenv("adatabase_url"),model_name="doubao-1-5-pro-32k-250115")
    # 先使用该函数,实现,创建, 初始化, 训练等等
    result = await intels.push_action_order(
            # demand='以下是一个代码模板, 我希望你做的是, 根据要求微调代码模板来构建代码```pythonfrom pydantic import BaseModel, Fieldfrom pro_craft import AsyncIntelinference_save_case = Falsemodel_name = "doubao-1-5-pro-256k-250115"inters = AsyncIntel(model_name = model_name)## modelsclass Chapter(BaseModel):    """    表示文档中的一个记忆卡片（章节）。    """    title: str = Field(..., description="记忆卡片的标题")    content: str = Field(..., description="记忆卡片的内容")class Document(BaseModel):    """    表示一个包含标题和多个记忆卡片的文档。    """    title: str = Field(..., description="整个文档的标题内容")    chapters: List[Chapter] = Field(..., description="文档中包含的记忆卡片列表")    ## modelsasync def amemory_card_merge(memory_cards: list[str]):    memoryCards_str, memoryCards_time_str = memoryCards2str(memory_cards)    input_data = memoryCards_str + "各记忆卡片的时间" + memoryCards_time_str        result = await inters.intellect_remove_format(        input_data= input_data,        prompt_id = "memorycard-merge",        version = None,        inference_save_case=inference_save_case,        OutputFormat = Document,        ExtraFormats=[Chapter],    )    return result```',
            demand="添加一些注释",
            prompt_id="zxf_code_template_2",
            action_type="to:1.8"
        )
    print(result)



async def test_work():
    intels = AsyncIntel(database_url=os.getenv("adatabase_url"),model_name="doubao-1-5-pro-32k-250115")
    # @intels.intellect_warp("zxf_code_template")
    # async def work_test(input_data, OutputFormat):
    #     print(input_data)
    await intels.create_main_database()
    # 期间不断使用这个函数进行配合
    input_data = "我要一个生成记忆卡片的程序,只包括文本即可"
    result = await intels.intellect(input_data=input_data,
                     output_format="",
                     prompt_id="zxf_code_template_2")
    
    print(result)


async def test_work2():
    intels = AsyncIntel(database_url=os.getenv("adatabase_url"),model_name="doubao-1-5-pro-32k-250115")
    
    @intels.intellect_warp("zxf_code_template_2")
    async def work_test(input_data, OutputFormat):
        print(input_data)
    input_data = "我要一个生成记忆卡片的程序, 记忆卡片包括标题(title), 内容(content), 时间(time), 标签(tag)"
    
    print(await work_test(input_data = input_data,
                          OutputFormat = ""))


llm2 = """
你是一个专业的Python代码修改和生成AI。你的任务是根据用户提供的代码模板文件和详细的修改指令文件，精确地对模板进行补充、完善和修改。

**你的目标是：**
1.  **严格遵循指令文件中的所有要求，尤其是针对特定 `BLOCK` 和 `PLACEHOLDER` 的指令和约束/示例。**
2.  **尽可能应用指令文件中提供的命名约定。**
3.  **仅修改指令明确要求修改的部分，模板中未被指令覆盖的固定部分必须保持不变。**
4.  **最终输出完整且可运行的Python代码。**

**输入格式:**
用户将以以下两个部分向你提供信息：

--- TEMPLATE_CODE_START ---
[原始的代码模板内容，其中包含 BLOCK_START/END 和 PLACEHOLDER/END_PLACEHOLDER 标记]
--- TEMPLATE_CODE_END ---

--- REQUEST_FILE_START ---
[一个结构化的指令文件，格式为 REQUEST_START/END，包含目标、命名约定和具体修改点]
--- REQUEST_FILE_END ---

**你的工作流程和生成原则:**

1.  **解析指令文件：**
    *   首先解析 `REQUEST_FILE_START` 中的所有内容，理解其 `目标`、`命名约定` 和 `具体修改点`。
    *   将 `具体修改点` 中的每个 `BLOCK` 和 `PLACEHOLDER` 指令及其 `约束/示例` 映射到模板代码中的对应位置。
2.  **处理模板代码：**
    *   逐行读取 `TEMPLATE_CODE_START` 中的模板代码。
    *   当遇到 `BLOCK_START` 或 `PLACEHOLDER` 标记时：
        *   查找指令文件中对应 `块名称` 的修改指令。
        *   **如果存在指令：**
            *   删除 `BLOCK_START` 和 `BLOCK_END` (或 `PLACEHOLDER` 和 `END_PLACEHOLDER`) 及其内部的原始内容（包括 `AI:` 注释）。
            *   用指令中提供的代码**替换**该区域。
            *   在替换的代码块的开始和结束位置，添加特殊的标记 `// AI_MODIFIED_START` 和 `// AI_MODIFIED_END` (如果只是新增内容，可以使用 `// AI_ADDED_START` 和 `// AI_ADDED_END`)。
            *   如果指令是要求删除某些内容，请用 `// AI_DELETED_LINE: [原始行内容]` 标记被删除的行。
        *   **如果不存在指令：**
            *   保留该 `BLOCK` 或 `PLACEHOLDER` 及其内部的原始内容（包括 `AI:` 注释和标记本身），不做任何改动。这允许模板中的可选部分在没有明确指令时保持原样。
    *   当遇到非标记的普通代码行时，保持其不变。
3.  **应用命名约定：**
    *   在生成或修改代码时，优先应用 `REQUEST_FILE_START` 中提供的 `命名约定`。
    *   **重要：** 命名约定只应影响由你**生成或修改**的代码部分（即 `AI_ADDED` 或 `AI_MODIFIED` 区域）。你不能随意修改模板中未被明确指令触及的固定代码部分的命名。
4.  **生成中间输出：**
    *   首先生成包含所有 `// AI_ADDED/MODIFIED/DELETED` 标记的完整代码。这有助于后续的自动化工具进行变更追踪和人工核查。
5.  **生成最终输出：**
    *   在生成中间输出后，进一步处理该代码，**移除所有 `// AI_ADDED/MODIFIED/DELETED` 类型的标记**。
    *   移除所有模板中遗留的 `BLOCK_START/END` 和 `PLACEHOLDER/END_PLACEHOLDER` 标记。
    *   保留所有的 Docstrings 和常规的代码注释。

**你的输出必须是最终的、清理后的完整 Python 代码文件内容。**

"""

from modusched.core import BianXieAdapter
bx = BianXieAdapter()
from pro_craft.utils import extract_
def write_code(request):
    result = bx.product(llm2 + request)
    print(result)
    python_code = extract_(result,r"python")
    with open("output.py","w",encoding="utf-8") as f:
        f.write(python_code)
    return "success" + " output.py generated."


import logging
import os
from logging.handlers import RotatingFileHandler, TimedRotatingFileHandler
import inspect
class Log:
    _instance = None
    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, console_level = logging.INFO, log_file_name="app.log"):
        self.Console_LOG_LEVEL = console_level
        self.log_file_name = log_file_name
        self.LOG_FILE_PATH = os.path.join("logs", log_file_name)
        os.makedirs(os.path.dirname(self.LOG_FILE_PATH), exist_ok=True)
        self.logger = self.get_logger()

    def get_logger(self):
        logger = logging.getLogger()
        logger.setLevel(logging.DEBUG)
        if not logger.handlers:
            # --- 4. 配置 Formatter (格式化器) ---
            # 以后有一个标准化的日志要使用logger 而非标的则使用super-log
            formatter = logging.Formatter(
                "%(asctime)s $ %(created)f $ %(levelname)s $ %(funcName)s $ :%(lineno)d $ %(pathname)s $ %(message)s||"
            )
            # --- 5. 配置 Handler (处理器) ---

            # 5.1 控制台处理器 (StreamHandler)
            console_handler = logging.StreamHandler()
            console_handler.setLevel(self.Console_LOG_LEVEL)  # 控制台只显示 INFO 及以上级别的日志
            console_handler.setFormatter(formatter)
            logger.addHandler(console_handler)

            # 文件系统
            ## 主日志本
            file_handler = RotatingFileHandler(  # RotatingFileHandler: 按文件大小轮转
                self.LOG_FILE_PATH,
                maxBytes=10 * 1024 * 1024,  # 10 MB # maxBytes: 单个日志文件的最大字节数 (例如 10MB)
                backupCount=10, # backupCount: 保留的旧日志文件数量
                encoding="utf-8",
            )
            file_handler.setLevel(logging.DEBUG)  # 文件中显示所有指定级别的日志
            file_handler.setFormatter(formatter)
            logger.addHandler(file_handler)

            ## 运行日志本
            file_handler_info = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_info.log'),
                maxBytes=10 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_info.setLevel(logging.INFO)  # 文件中显示所有指定级别的日志
            file_handler_info.setFormatter(formatter)
            logger.addHandler(file_handler_info)

            ## 错误日志本
            file_handler_warning = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_err.log'),
                maxBytes=10 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_warning.setLevel(logging.WARNING)  # 文件中显示所有指定级别的日志
            file_handler_warning.setFormatter(formatter)
            logger.addHandler(file_handler_warning)

            ## 指定日志本 
            file_handler_super = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_s.log'),
                maxBytes=5 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_super.setLevel(logging.CRITICAL)  # 文件中显示所有指定级别的日志
            file_handler_super.setFormatter(formatter)
            logger.addHandler(file_handler_super)

            # class ConfigFilter(logging.Filter):
            #     # 初始化的方案
            #     def __init__(self, config_value, name=''):
            #         super().__init__(name)
            #         self.config_value = config_value

            #     def filter(self, record):
            #         record.config_info = self.config_value # 添加到LogRecord
            #         return True


            # class CustomFilter(logging.Filter):
            #     def filter(self, record):
            #         # 添加自定义的用户ID
            #         print(record,'recordrecordrecordrecordrecord')
            #         record.user_id = os.getenv("CURRENT_USER_ID", "anonymous")
            #         # 添加会话ID
            #         # record.session_id = "SESSION_XYZ123" # 实际应用中可能从request或全局变量获取
            #         frame = inspect.currentframe()
            #         info = inspect.getframeinfo(frame)
            #         # logger.error(f"Function name: {info.function} : {e}")
            #         record.session_id = f"Function name: {info.function}"
                    
            #         return True # 必须返回True才能继续处理该日志记录

            # # 将自定义过滤器添加到处理器或记录器
            # logger.addFilter(CustomFilter())
            # handler.addFilter(CustomFilter()) # 也可以加到handler上

            logger.info("这是一个包含自定义信息的日志")
        return logger
    


# LLM1

import os
from typing import Literal
# from tavily import TavilyClient
from deepagents import create_deep_agent
from .tools import search_template_by_text, get_template_details, ask_user_for_clarification, generate_request_file


# System prompt to steer the agent to be an expert researcher
research_instructions = """你是一个高级代码生成助手 Agent，你的主要职责是帮助用户通过模板快速生成代码。

**你的目标是：**
1.  **精确理解用户对代码功能和结构的自然语言描述。**
2.  **在现有模板库中，智能地找到最符合用户需求的模板。**
3.  **如果找不到高匹配度的模板，引导用户创建新模板。**
4.  **基于选定的模板和用户需求，生成一个结构化、可执行的 `REQUEST_START/END` 指令文件，用于后续的代码生成。**
5.  **在必要时与用户进行交互，澄清需求或提供建议。**

**你的可用工具:**

1.  **`search_template_by_text(query: str, top_k: int = 5) -> List[Dict[str, Any]]`**
    *   **描述:** 根据自然语言查询在模板库中进行语义搜索（使用 Qdrant 向量检索），返回最相关的 `top_k` 个模板的摘要信息。
    *   **返回字段示例:** `[{'template_id': 'uuid', 'template_name': 'str', 'description': 'str', 'match_score': 'float'}]`
    *   **何时使用:** 当你需要根据用户需求找到合适的代码模板时。

2.  **`get_template_details(template_id: str) -> Dict[str, Any]`**
    *   **描述:** 根据 `template_id` 从数据库中获取模板的完整详细信息，包括模板代码、推断出的命名约定和使用建议。
    *   **返回字段示例:** `{'template_id': 'uuid', 'template_name': 'str', 'template_code': 'str', 'suggested_naming_conventions': 'json', 'usage_guidance': 'str'}`
    *   **何时使用:** 当你已经选择了一个模板，需要其详细内容来生成指令时。

3.  **`generate_request_file(template_code: str, user_request_details: Dict[str, Any], naming_conventions: Dict[str, Any]) -> str`**
    *   **描述:** 根据选定的模板代码、解析后的用户需求（结构化形式）和模板的命名约定，生成符合 `REQUEST_START/END` 格式的指令文件。
    *   **何时使用:** 当你已经确定了模板，并且充分理解了用户需求的所有细节，准备生成最终指令时。

4.  **`ask_user_for_clarification(question: str) -> str`**
    *   **描述:** 当你对用户需求有疑问，或需要用户做选择（例如在多个匹配模板中选择一个）时，使用此工具向用户提问。
    *   **返回:** 用户的回答。
    *   **何时使用:** 任何需要用户输入或确认的场景。

**你的工作流程:**

1.  **接收用户需求：** 用户会提供一个自然语言描述。
2.  **初步理解与模板搜索：**
    *   首先使用 `search_template_by_text` 工具，以用户需求的概要作为 `query`，找到 `top_k` 个最相关的模板。
    *   分析搜索结果中的 `match_score` 和 `description`，评估匹配度。
3.  **决策点 - 模板匹配：**
    *   **高匹配度：** 如果存在一个或少数几个模板的 `match_score` 显著高，且 `description` 与用户需求高度吻合：
        *   使用 `get_template_details` 获取该模板的完整信息。
        *   进入 **需求细化与指令生成** 阶段。
    *   **中等匹配度 / 多个相似匹配：** 如果有多个模板得分接近，或没有一个模板完美匹配：
        *   使用 `ask_user_for_clarification` 工具，向用户展示这些模板的 `template_name` 和 `description`，并询问用户希望选择哪一个，或者是否希望在此基础上进行调整。
        *   根据用户反馈，决定是选择一个模板还是引导用户创建新模板。
    *   **低匹配度 / 无匹配：** 如果没有找到任何合适的模板（例如，所有 `match_score` 都很低）：
        *   使用 `ask_user_for_clarification` 工具，告知用户未能找到合适的模板，并询问用户是否希望提供多个示例代码，以便使用 LLM 0 创建一个新的模板。
        *   如果用户同意，引导用户进入 LLM 0 的流程。
4.  **需求细化与指令生成 (基于选定模板):**
    *   一旦确定了模板，仔细解析用户需求的每个细节，并将其映射到选定模板中的 `BLOCK` 和 `PLACEHOLDER`。
    *   考虑模板的 `suggested_naming_conventions`，并尝试将其整合到生成的指令中。
    *   如果用户需求与模板的某个 `BLOCK` 或 `PLACEHOLDER` 不兼容，或用户没有提供足够的细节来填充某个区域，使用 `ask_user_for_clarification` 向用户提问。
    *   当所有必要信息都已获取且明确无误时，使用 `generate_request_file` 工具生成最终的 `REQUEST_START/END` 指令文件。
5.  **输出最终指令：** 将 `generate_request_file` 的输出返回给系统，以便进行下一步的代码生成。

**交互约束:**
*   除非使用 `ask_user_for_clarification` 工具，否则不要直接与用户对话。
*   始终以使用工具作为首选行动。
*   保持你的回复简洁、直接，聚焦于完成任务。

**示例用户需求:**
"我需要一个API来发送用户消息。路径是 `/send`，POST 方法。输入模型叫 `SendMessageRequest`，包含 `user_id` (UUID格式) 和 `message_content` (字符串，最大500字)。输出模型叫 `SendMessageResponse`，继承 `CommonResponseModel`，额外包含 `message_id`。它会调用 `message_service_manager.send_message(user_id, content)`。如果 `user_id` 无效，应该返回 400 错误。"
"""
from langchain.chat_models import init_chat_model

model = init_chat_model(
            model="openai:gemini-2.5-flash-preview-05-20-nothinking",
            api_key=os.getenv("BIANXIE_API_KEY"),
            base_url=os.getenv("BIANXIE_BASE"),
        )

agent = create_deep_agent(
    model=model,   
    tools=[search_template_by_text, get_template_details, ask_user_for_clarification, generate_request_file],
    system_prompt=research_instructions
)


if __name__ == "__main__":
    result = agent.invoke({"messages": [{"role": "user", "content": "hello"}]})

    print(result["messages"][-1].content)


# --- 假设的依赖项 ---
# Qdrant 客户端
from qdrant_client import QdrantClient, models # 你需要根据实际情况导入 Qdrant 客户端
qdrant_client = QdrantClient(host="localhost", port=6333) # 示例Qdrant客户端初始化


from typing import Any, List
from llama_index.core.bridge.pydantic import PrivateAttr
from llama_index.core.embeddings import BaseEmbedding
from volcenginesdkarkruntime import Ark


class VolcanoEmbedding(BaseEmbedding):
    _model = PrivateAttr()
    _ark_client = PrivateAttr()
    _encoding_format = PrivateAttr()

    def __init__(
        self,
        model_name: str = "doubao-embedding-text-240715",
        api_key: str = "",
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)
        self._ark_client = Ark(api_key=api_key)
        self._model = model_name
        self._encoding_format = "float"

    @classmethod
    def class_name(cls) -> str:
        return "ark"

    def _get_query_embedding(self, query: str) -> List[float]:
        """
        获取查询字符串的 embedding。
        通常查询和文档使用相同的 embedding 模型。
        """

        resp = self._ark_client.embeddings.create(
            model=self._model,
            input=[query],
            encoding_format=self._encoding_format,
        )
        return resp.data[0].embedding

    def _get_text_embedding(self, text: str) -> List[float]:
        """
        获取单个文档字符串的 embedding。
        """
        resp = self._ark_client.embeddings.create(
            model=self._model,
            input=[text],
            encoding_format=self._encoding_format,
        )
        return resp.data[0].embedding

    def _get_text_embeddings(self, texts: List[str]) -> List[List[float]]:
        """
        批量获取文档字符串的 embedding。
        如果你的火山模型支持批量推理，强烈建议实现此方法以提高效率。
        否则，它可以简单地循环调用 _get_text_embedding。
        """
        resp = self._ark_client.embeddings.create(
            model=self._model,
            input=texts,
            encoding_format=self._encoding_format,
        )
        return [i.embedding for i in resp.data]

    async def _aget_query_embedding(self, query: str) -> List[float]:
        return self._get_query_embedding(query)

    async def _aget_text_embedding(self, text: str) -> List[float]:
        return self._get_text_embedding(text)

if __name__ == "__main__":

    embedding_model = VolcanoEmbedding(
        model_name = "doubao-embedding-text-240715",
        api_key = "39ad310a-c6f7-4d66-962e-1fbfa7e6edf1"
    )


import pytest
from pro_craft.code_helper.codermanager import Paper, ProgramChart
from pro_craft.code_helper.codermanager import CoderHelper
from dotenv import load_dotenv
load_dotenv(".env", override=True)

def test_min_edit():
    x = CoderHelper().min_edit("print(hello world)","写成中文")
    print(x)


def test_free_function():
    """
    result2 = highly_flexible_function(x="帮我计算 1加1")
    print(f"示例2 执行结果: {result2}")

    """
    x = CoderHelper().free_function(
        function="帮我将 日期 2025/12/03 向前回退12天", 
        data_str="2025/12/03", 
        days=12)
    print(x,"result->")

def test_free_function_advanced():
    return "1234"





def test_draw_paper():
    """
    在一个固定画板上编辑文本
    """
    content = """我在凉山州做项目期间，曾替客户背过一次“黑锅”。那次是我们接处警的项目刚刚上线，因为项目是在凉山州做的，所以我们在西昌市驻扎。当地有两个公安局，一个是州公安局，一个是市公安局。上线当天，我们原定在州公安局开新闻发布会，市公安局这边会同步进行。但就在当天早上，我被客户叫到了市公安局，本该去参加发布会的我被临时调动。客户告诉我，市公安局的大领导、局长可能会来问些问题，让我再去给他讲讲我们的系统。于是我直接去了市公安局，没有去参加发布会。\n\n当时叫我去的是客户那边的对接人，一位指挥长。在他们开新闻发布会的时候，我们就在后面聊天，并给他讲一些系统的事情，因为我们关系都比较熟了。发布会结束后，公安局的局长说要让我调一份接警的录音出来。这事儿是因为他们下面有一个派出所警员在接警过程中，对接警员的态度不好，局长比较生气，说要调出证据去处罚他，认为他既然对自己的同志态度都不好，那对待民众的态度肯定会更不好。他只要那份录音文件，让我从我们的系统里导出来。\n\n局长要在他们全市的公安局领导面前播放这份录音，但是他们播放的设备不能直接登录我们的系统，所以我需要把录音文件导出来，再插到他们播放的电脑上去播放。我找到U盘后，因为我在那块待的时间比较长，比较熟，就想把音频文件直接导到他们的电脑上，然后播放出来。"""
    pp = Paper(content)
    pp.talk(' 帮我将上面文字中关于u盘的内容删除')
    print(pp.content)

def test_paper():
    
    # 在一张白纸上作画
    pp = Paper("你好 世界")
    x = pp.talk('你好')
    print(x)







#############
# 1 编写自动化构建web服务
# 2 编写自动化构建框架服务
# 3 编写自动化实现低难度框架
# 4 


#############


from prompt_writing_assistant.utils import super_print
from prompt_writing_assistant.code_helper.web import edit_server
from prompt_writing_assistant.code_helper.web import gener_post_server,gener_post_server_multi,gener_get_server

import pytest


# 格式输出不稳定

def test_edit_server():
    """
    1 uv run python -m temp_web # 搭建起虚拟服务
    2 运行, 写入服务更新, 然后获得curl 命令, 验证是否通行

    优势是: 结合了本地化, 可以同步的测试服务是否可用, 同时生成curl 命令
    缺点是: 冗长, 繁琐, 单一操作
    """
    # uv run python -m temp_web
    ask = '''
# 记忆合并
class Memory_card_list_Request(BaseModel):
    memory_cards: list[str] = Field(..., description="要评分的记忆卡片内容")

@app.post("/memory_card/merge")
async def memory_card_merge_server(request: Memory_card_list_Request):
    """
    记忆卡片质量评分
    接收一个记忆卡片内容字符串，并返回其质量评分。
    """
    result = memory_card_merge(memory_cards=request.memory_cards)
    return {
        "message": "memory card merge successfully",
        "result": result
    }

帮我修改为post方法
'''
    
    out_curl = edit_server(ask,output_py = 'temp_web.py')
    super_print(out_curl,"test_edit_server_curl")




def test_get_server():
    """
    自动生成get 方法

    优点: 可控性强, 靠模板制定函数
    缺点: 人类介入程度高, 单一操作
    """
    code = '''
    def brief(self,memory_cards:list[str])->str:
        """
        数字分身介绍
        """
        return {"title":"我的数字分身标题","content":"我的数字分身简介"}
    '''
    result = gener_get_server(prompt=code)
    super_print(result,'gener_get_server')



def test_post_server():
    """
    自动生成get 方法

    优点: 可控性强, 靠模板制定函数
    缺点: 人类介入程度高, 单一操作
    """
    code = '''
    def brief(self,memory_cards:list[str])->str:
        """
        数字分身介绍
        """
        return {"title":"我的数字分身标题","content":"我的数字分身简介"}
    '''
    result = gener_post_server(prompt=code)
    super_print(result,'post_server')



def test_post_server_multi():
    """
    自动生成get 方法

    优点: 可控性强, 靠模板制定函数, 多数操作, 有格式监控
    缺点: 人类介入程度高
    """
    code = '''
class Memo():
    def __init__(self):
        """
        初始化
        """
        
    def build_from_datas(datas:dict)->None:
        """
        构建数据
        """
        
    def get_info_from_tree(self, prompt:str)-> str:
        """
        获取信息
        """
        
    def eval_(self)-> str:
        """
        这是一个长时间运行的同步操作
        """
        
    async def aeval_(self)-> str:
        """
        这是一个长时间运行的异步操作, 作用同eval_
        """
        
    def get_eval(self)-> str:
        """
        获取eval 的执行结果
        """
    '''
    result = gener_post_server_multi(prompt=code)
    super_print(result,'post_server')





from sqlalchemy import Column, Integer, String, Text, DateTime, text, UniqueConstraint, Boolean, func, Float, Double
from sqlalchemy.orm import declarative_base

from datetime import datetime, timedelta

FileBase = declarative_base()


# template_id (PK)
#  template_name (e.g., "FastAPI Router Endpoint")
#  description (e.g., "FastAPI服务路由器中的单个API端点，包含Pydantic校验、异步调用、Try-Except错误处理和日志记录。")
#  template_code (实际的模板代码字符串)
#  suggested_naming_conventions (JSONB 或文本)
#  usage_guidance (文本)
#  language (e.g., "Python")
#  framework (e.g., "FastAPI")
#  version
#  created_by
#  created_at
#  last_updated_at


from sqlalchemy import Column, Integer, String, Text, DateTime, text, UniqueConstraint, Boolean, func, Float, Double
from sqlalchemy.orm import declarative_base

from datetime import datetime, timedelta

Base = declarative_base()

class CodeTemplate(Base):
    __tablename__ = 'code_template'
    template_id = Column(String(255), primary_key=True, nullable=False, comment="自增")
    description = Column(Text, nullable=False, comment="Detailed description of the template")
    template_code = Column(Text, nullable=False, comment="The actual code of the template")
    version = Column(Integer, nullable=True, comment="Template version")
    # UniqueConstraint('template_name', name='uq_template_name') # 如果 template_name 应该是唯一的
